/*
 *  Copyright 2019-2025 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package com.fuyuanshen.modules.system.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import com.fuyuanshen.config.properties.FileProperties;
import com.fuyuanshen.constants.RoleConstants;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.exception.EntityExistException;
import com.fuyuanshen.modules.security.service.OnlineUserService;
import com.fuyuanshen.modules.security.service.UserCacheManager;
import com.fuyuanshen.modules.system.constant.UserConstants;
import com.fuyuanshen.modules.system.domain.Job;
import com.fuyuanshen.modules.system.domain.Role;
import com.fuyuanshen.modules.system.domain.User;
import com.fuyuanshen.modules.system.domain.dto.UserQueryCriteria;
import com.fuyuanshen.modules.system.domain.query.UserQuery;
import com.fuyuanshen.modules.system.domain.vo.ConsumerVo;
import com.fuyuanshen.modules.system.mapper.RoleMapper;
import com.fuyuanshen.modules.system.mapper.UserJobMapper;
import com.fuyuanshen.modules.system.mapper.UserMapper;
import com.fuyuanshen.modules.system.mapper.UserRoleMapper;
import com.fuyuanshen.modules.system.service.UserService;
import com.fuyuanshen.utils.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author Zheng Jie
 * @date 2018-11-23
 */
@Service
@RequiredArgsConstructor
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    private final UserMapper userMapper;
    private final UserJobMapper userJobMapper;
    private final UserRoleMapper userRoleMapper;
    private final FileProperties properties;
    private final RedisUtils redisUtils;
    private final UserCacheManager userCacheManager;
    private final OnlineUserService onlineUserService;
    private final RoleMapper roleMapper;


    @Override
    public PageResult<User> queryAll(UserQueryCriteria criteria, Page<Object> page) {
        criteria.setOffset(page.offset());
        List<User> users = userMapper.findAll(criteria);
        users = users.stream().map(user -> {
            if (user.getAdmin() == 1) {
                user.setIsAdmin(true);
            }
            return user;
        }).collect(Collectors.toList());

        Long total = userMapper.countAll(criteria);
        return PageUtil.toPage(users, total);
    }

    @Override
    public List<User> queryAll(UserQueryCriteria criteria) {
        return userMapper.findAll(criteria);
    }


    /**
     * 分页查询客户
     *
     * @param criteria
     * @param page
     * @return
     */
    @Override
    public PageResult<ConsumerVo> queryCustomers(UserQueryCriteria criteria, Page<User> page) {
        criteria.setOffset(page.offset());
        List<User> users = userMapper.findCustomers(criteria);
        List<ConsumerVo> consumerVoList = BeanUtil.copyToList(users, ConsumerVo.class);
        Long total = userMapper.countCustomers(criteria);
        return PageUtil.toPage(consumerVoList, total);
    }

    /**
     * 分页查询客户
     *
     * @param criteria
     * @param page
     * @return
     */
    public PageResult<ConsumerVo> queryCustomers1(UserQueryCriteria criteria, Page<User> page) {
        criteria.setOffset(page.offset());

        // Step 1: 查询符合搜索条件的用户（扁平）
        List<User> users = userMapper.findCustomers(criteria);

        if (users.isEmpty()) {
            return PageUtil.toPage(Collections.emptyList(), 0L);
        }

        // Step 2: 提取这些用户的 ID
        Set<Long> matchedUserIds = users.stream().map(User::getId).collect(Collectors.toSet());

        // Step 3: 查询这些用户及其所有祖先
        List<User> allRelevantUsers = userMapper.findUsersWithAncestorsByIds(matchedUserIds);

        // Step 4: 转换为 ConsumerVo 列表
        List<ConsumerVo> consumerVos = allRelevantUsers.stream().map(this::convertToConsumerVo).toList();

        // Step 5: 构建树形结构
        List<ConsumerVo> treeStructure = buildUserTreeList(consumerVos);

        // Step 6: 返回树形结构 + 总数
        Long total = userMapper.countCustomers(criteria);
        return PageUtil.toPage(treeStructure, total);
    }


    /**
     * 根据用户ID构建用户树
     */
    private List<ConsumerVo> buildUserTreeList(List<ConsumerVo> allUsers) {
        Map<Long, ConsumerVo> userMap = new HashMap<>();
        List<ConsumerVo> rootNodes = new ArrayList<>();

        for (ConsumerVo user : allUsers) {
            userMap.put(user.getId(), user);
        }

        for (ConsumerVo user : allUsers) {
            Long parentId = user.getPid();
            if (parentId == null || parentId.equals(0L)) {
                rootNodes.add(user);
            } else {
                ConsumerVo parent = userMap.get(parentId);
                if (parent != null) {
                    if (parent.getChildren() == null) {
                        parent.setChildren(new ArrayList<>());
                    }
                    parent.getChildren().add(user);
                }
            }
        }

        return rootNodes;
    }

    private ConsumerVo convertToConsumerVo(User user) {
        ConsumerVo vo = new ConsumerVo();
        vo.setId(user.getId());
        vo.setUsername(user.getUsername());
        vo.setNickName(user.getNickName());
        vo.setEnabled(user.getEnabled());
        vo.setPid(user.getPid()); // 必须包含 pid
        return vo;
    }

    /**
     * 查询所有客户
     *
     * @param criteria
     * @return
     */
    @Override
    public List<User> queryAllCustomers(UserQueryCriteria criteria) {
        List<User> users = userMapper.queryAllCustomers(criteria);
        return users;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public User findById(long id) {
        String key = CacheKey.USER_ID + id;
        User user = redisUtils.get(key, User.class);
        if (user == null) {
            user = getById(id);
            redisUtils.set(key, user, 1, TimeUnit.DAYS);
        }
        return user;
    }


    /**
     * 新增用户
     *
     * @param resources /
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void create(User resources) {
        // resources.setDeptId(resources.getDept().getId());
        if (userMapper.findByUsername(resources.getUsername()) != null) {
            throw new EntityExistException(User.class, "账号", resources.getUsername());
        }
        if (userMapper.findByEmail(resources.getEmail()) != null) {
            throw new EntityExistException(User.class, "邮箱", resources.getEmail());
        }
        if (userMapper.findByPhone(resources.getPhone()) != null) {
            throw new EntityExistException(User.class, "电话号码", resources.getPhone().toString());
        }

        User currentUser = this.findByName(SecurityUtils.getCurrentUsername());
        resources.setPid(currentUser.getId());
        if (currentUser.getUserLevel() != null && currentUser.getUserLevel() >= UserConstants.MAX_USER_LEVEL) {
            throw new RuntimeException(UserConstants.USER_LEVEL_EXCEED_LIMIT_MESSAGE);
        }
        resources.setUserLevel((byte) (currentUser.getUserLevel() + 1));
        save(resources);
        // 保存用户岗位
//        userJobMapper.insertData(resources.getId(), resources.getJobs());
        // 保存用户角色
        userRoleMapper.insertData(resources.getId(), resources.getRoles());
    }


    /**
     * 新增客户
     *
     * @param user /
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addCustomer(User user) {
        // if (userMapper.findByNickName(user.getNickName()) != null) {
        //     throw new EntityExistException(User.class, "名称", user.getNickName());
        // }

        if (userMapper.findByUsername(user.getUsername()) != null) {
            throw new BadRequestException("用户名不符合基本安全要求！！！");
        }

        User currentUser = userMapper.findByUsername(SecurityUtils.getCurrentUsername());

        UserQuery userQuery = new UserQuery();
        userQuery.setPid(currentUser.getId());
        userQuery.setNickName(user.getNickName());

        if (userMapper.selectByQueryOne(userQuery) != null) {
            throw new BadRequestException("客户已存在！！！");
        }
        // userQuery = new UserQuery();;
        // userQuery.setUsername(user.getUsername());
        // userQuery.setPassword(user.getPassword());
        // if (userMapper.selectByQueryOne(userQuery) != null) {
        //     throw new BadRequestException("密码或者用户名不符合安全规范！！！");
        // }

//        user.setDeptId(user.getDept().getId());
        user.setPid(currentUser.getId());
        user.setEnabled(true);
        if (currentUser.getUserLevel() != null && currentUser.getUserLevel() >= UserConstants.MAX_USER_LEVEL) {
            throw new RuntimeException(UserConstants.USER_LEVEL_EXCEED_LIMIT_MESSAGE);
        }

        user.setUserLevel((byte) (currentUser.getUserLevel() + 1));
        // 租户id
        long tenantId = 0L;
        // if (currentUser.getTenantId() == null || currentUser.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
        //     tenantId = IdUtil.getSnowflakeNextId();
        // } else {
        //     tenantId = currentUser.getTenantId();
        // }
        tenantId = IdUtil.getSnowflakeNextId();
        user.setTenantId(tenantId);
        save(user);
        // 保存用户角色
//        userRoleMapper.insertData(user.getId(), user.getRoles());

        // 自动绑定默认角色
        Set<Role> defaultRoles = new HashSet<>();
        Role defaultRole = roleMapper.findByName(RoleConstants.ROLE_CUSTOMER);
        if (defaultRole == null) {
            throw new RuntimeException("未找到默认角色");
        }
        defaultRoles.add(defaultRole);
        // 绑定角色
        userRoleMapper.insertData(user.getId(), defaultRoles);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(User resources) throws Exception {
        User user = getById(resources.getId());
        User user1 = userMapper.findByUsername(resources.getUsername());
        User user2 = userMapper.findByEmail(resources.getEmail());
        User user3 = userMapper.findByPhone(resources.getPhone());
        if (user1 != null && !user.getId().equals(user1.getId())) {
            throw new EntityExistException(User.class, "账号", resources.getUsername());
        }
        if (user2 != null && !user.getId().equals(user2.getId())) {
            throw new EntityExistException(User.class, "邮箱", resources.getEmail());
        }
        if (user3 != null && !user.getId().equals(user3.getId())) {
            throw new EntityExistException(User.class, "电话号码", resources.getPhone().toString());
        }
        // 如果用户的角色改变
        if (!resources.getRoles().equals(user.getRoles())) {
            redisUtils.del(CacheKey.DATA_USER + resources.getId());
            redisUtils.del(CacheKey.MENU_USER + resources.getId());
            redisUtils.del(CacheKey.ROLE_AUTH + resources.getId());
            redisUtils.del(CacheKey.ROLE_USER + resources.getId());
        }
        // 修改部门会影响 数据权限
        if (!Objects.equals(resources.getDept(), user.getDept())) {
            redisUtils.del(CacheKey.DATA_USER + resources.getId());
        }
        // 如果用户被禁用，则清除用户登录信息
        if (!resources.getEnabled()) {
            onlineUserService.kickOutForUsername(resources.getUsername());
        }
        user.setDeptId(resources.getDept().getId());
        user.setUsername(resources.getUsername());
        user.setEmail(resources.getEmail());
        user.setEnabled(resources.getEnabled());
        user.setRoles(resources.getRoles());
        user.setDept(resources.getDept());
        user.setJobs(resources.getJobs());
        user.setPhone(resources.getPhone());
        user.setNickName(resources.getNickName());
        user.setGender(resources.getGender());
        saveOrUpdate(user);
        // 清除缓存
        delCaches(user.getId(), user.getUsername());
        // 更新用户岗位
        if (CollectionUtil.isNotEmpty(resources.getJobs())) {
            userJobMapper.deleteByUserId(resources.getId());
            userJobMapper.insertData(resources.getId(), resources.getJobs());
        }
        // 更新用户角色
        userRoleMapper.deleteByUserId(resources.getId());
        userRoleMapper.insertData(resources.getId(), resources.getRoles());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCustomer(User resources) throws Exception {
        Timestamp timestamp = new Timestamp(System.currentTimeMillis());
        User user = getById(resources.getId());
        if (user == null) {
            throw new BadRequestException("您要修改的客户不存在");
        }
        User user1 = userMapper.findByUsername(resources.getUsername());
        if (user1 != null && !user.getId().equals(user1.getId())) {
            throw new EntityExistException(User.class, "账号", resources.getUsername());
        }
        // 如果用户被禁用，则清除用户登录信息
        if (resources.getEnabled() != null && resources.getEnabled() == false) {
            onlineUserService.kickOutForUsername(resources.getUsername());
        }
        user.setUsername(resources.getUsername());
        user.setNickName(resources.getNickName());
        user.setEnabled(resources.getEnabled());
        user.setUpdateTime(timestamp);
        saveOrUpdate(user);
        // 清除缓存
        delCaches(user.getId(), user.getUsername());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCenter(User resources) {
        User user = getById(resources.getId());
        User user1 = userMapper.findByPhone(resources.getPhone());
        if (user1 != null && !user.getId().equals(user1.getId())) {
            throw new EntityExistException(User.class, "phone", resources.getPhone().toString());
        }
        user.setNickName(resources.getNickName());
        user.setPhone(resources.getPhone());
        user.setGender(resources.getGender());
        saveOrUpdate(user);
        // 清理缓存
        delCaches(user.getId(), user.getUsername());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Set<Long> ids) {
        for (Long id : ids) {
            // 清理缓存
            User user = getById(id);
            if (user != null) {
                delCaches(user.getId(), user.getUsername());
            }
        }
        userMapper.deleteBatchIds(ids);
        // 删除用户岗位
        /*userJobMapper.deleteByUserIds(ids);
        // 删除用户角色
        userRoleMapper.deleteByUserIds(ids);*/
    }

    @Override
    public User findByName(String userName) {
        return userMapper.findByUsername(userName);
    }

    @Override
    public User getLoginData(String userName) {
        return userMapper.findByUsername(userName);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePass(String username, String pass) {
        userMapper.updatePass(username, pass, new Date());
        flushCache(username);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void resetPwd(Set<Long> ids, String pwd) {
        List<User> users = userMapper.selectBatchIds(ids);
        // 清除缓存
        users.forEach(user -> {
            // 清除缓存
            flushCache(user.getUsername());
            // 强制退出
            onlineUserService.kickOutForUsername(user.getUsername());
        });
        // 重置密码
        userMapper.resetPwd(ids, pwd);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map<String, String> updateAvatar(MultipartFile multipartFile) {
        // 文件大小验证
        FileUtil.checkSize(properties.getAvatarMaxSize(), multipartFile.getSize());
        // 验证文件上传的格式
        String image = "gif jpg png jpeg";
        String fileType = FileUtil.getExtensionName(multipartFile.getOriginalFilename());
        if (fileType != null && !image.contains(fileType)) {
            throw new BadRequestException("文件格式错误！, 仅支持 " + image + " 格式");
        }
        User user = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        String oldPath = user.getAvatarPath();
        File file = FileUtil.upload(multipartFile, properties.getPath().getAvatar());
        user.setAvatarPath(Objects.requireNonNull(file).getPath());
        user.setAvatarName(file.getName());
        saveOrUpdate(user);
        if (StringUtils.isNotBlank(oldPath)) {
            FileUtil.del(oldPath);
        }
        @NotBlank String username = user.getUsername();
        flushCache(username);
        return new HashMap<String, String>(1) {{
            put("avatar", file.getName());
        }};
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateEmail(String username, String email) {
        userMapper.updateEmail(username, email);
        flushCache(username);
    }

    @Override
    public void download(List<User> users, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (User user : users) {
            List<String> roles = user.getRoles().stream().map(Role::getName).collect(Collectors.toList());
            Map<String, Object> map = new LinkedHashMap<>();
            map.put("用户名", user.getUsername());
            map.put("角色", roles);
            map.put("部门", user.getDept().getName());
            map.put("岗位", user.getJobs().stream().map(Job::getName).collect(Collectors.toList()));
            map.put("邮箱", user.getEmail());
            map.put("状态", user.getEnabled() ? "启用" : "禁用");
            map.put("手机号码", user.getPhone());
            map.put("修改密码的时间", user.getPwdResetTime());
            map.put("创建日期", user.getCreateTime());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

    /**
     * 清理缓存
     *
     * @param id /
     */
    public void delCaches(Long id, String username) {
        redisUtils.del(CacheKey.USER_ID + id);
        flushCache(username);
    }

    /**
     * 清理 登陆时 用户缓存信息
     *
     * @param username /
     */
    private void flushCache(String username) {
        userCacheManager.cleanUserCache(username);
    }


    /**
     * 获取当前用户的子用户
     *
     * @param currentUserId
     * @return
     */
    @Override
    public List<User> findUserTree(Long currentUserId) {
        return userMapper.findUserTree(currentUserId);
    }


}
